/**
 * Created by Grigorii Shovkoplias on 23.03.2016.
 */

import java.io.*;
import java.util.*;

public class TestGen {

    private void gen() throws Exception {
        new File("../tests").mkdir();
        new File("../preliminary").mkdir();
        desc = new PrintWriter("../tests.desc");


        done = 0;
        genSamples();

        //-------------------------first group-------------------------------------------------
        subtask = "subtask1";

        genManual();
        for (int i = 10; i <= MAXN1; i *= 10) {
            genRand(gand(i));
            genRand(gand(i));
        }

        for (int i = 0; i < 5; i++) {
            genManual(MAXN1, MAXN1, rand(MAXN1) + 1, rand(MAXN1) + 1);
        }

        desc.println(String.format("comment_subtask%d=\"n &lt;= %d\"", 1, MAXN1));
        desc.println(String.format("bonus_subtask%d=%d", 1, 25));

        //-------------------------second group-------------------------------------------------

        subtask = "subtask2";

        for (int i = MAXN1 * 10; i <= MAXN2; i *= 10) {
            genRand(gand(i));
            genRand(gand(i));
        }

        for (int i = 0; i < 5; i++) {
            genManual(MAXN2, MAXN2, rand(MAXN2) + 1, rand(MAXN2) + 1);
        }

        desc.println(String.format("comment_subtask%d=\"n &lt;= %d\"", 2, MAXN2));
        desc.println(String.format("bonus_subtask%d=%d", 2, 27));

        //-------------------------third group-------------------------------------------------

        subtask = "subtask3";

        for (int i = MAXN2 * 10; i <= MAXN3; i *= 10) {
            genRand(gand(i));
            genRand(gand(i));
        }

        for (int i = 0; i < 5; i++) {
            genManual(MAXN3, MAXN3, rand(MAXN3) + 1, rand(MAXN3) + 1);
        }

        desc.println(String.format("comment_subtask%d=\"n &lt;= %d\"", 3, MAXN3));
        desc.println(String.format("bonus_subtask%d=%d", 3, 48));



        desc.close();

    }

    public static void main(String[] args) throws Exception {
        new TestGen().gen();
    }

    private int done = 0;
    private PrintWriter desc;
    private final Random rnd = new Random(3289214075201L);
    private final int MAXN1 = 1000;
    private final int MAXN2 = 100_000;
    private final int MAXN3 = 1_000_000_000;

    private int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    private char randChar() {
        return (char) (rand(26) + 'a');
    }

    private int gand(int n) {
        return rand(n / 2) + n / 2 + 1;
    }

    private boolean coin() {
        return rnd.nextBoolean();
    }

    private long rand(long n) {
        if (n < 1)
            return 0;
        long res = rand(Integer.MAX_VALUE);
        res <<= 32;
        res += rand(Integer.MAX_VALUE);
        res %= n;
        return res;
    }


    private void print(int n, int m, int x, int y) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", done));
        out.println(n + " " + m);
        out.println(x + " " + y);
        out.close();
    }


    private String getPath() {
        return String.format("../tests/%02d", done);
    }

    private void printDesc(String path, String folder, String subtask, int points, String comment) throws IOException {
        desc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, points, comment));
    }

    private String subtask;

    private void printDescT(String comment) throws IOException {
        printDesc(getPath(), "tests", subtask, 0, comment);
    }

    //begin generators

    private void genSamples() throws IOException {
        PrintWriter out;
        String path;
        path = "../preliminary/01";
        out = new PrintWriter(path);
        out.println("7 6");
        out.println("3 4");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 1");
        path = "../preliminary/02";
        out = new PrintWriter(path);
        out.println("2 2");
        out.println("1 1");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 2");
        path = "../preliminary/03";
        out = new PrintWriter(path);
        out.println("2 2");
        out.println("1 2");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 3");
    }


    private void shuffle(int[] a) {
        for (int i = 0; i < a.length; i++) {
            int j = rand(a.length);
            int tmp = a[j];
            a[j] = a[i];
            a[i] = tmp;
        }
    }

    //begin generators

    private void genManual() throws IOException {
        print(1, 1, 1, 1);
        printDescT("1 1 1 1");
        print(5, 5, 1, 1);
        printDescT("5 5 1 1");
        print(4, 4, 4, 4);
        printDescT("4 4 4 4");
        print(5, 5, 3, 3);
        printDescT("5 5 3 3");
    }

    private void genRand(int maxn) throws IOException {
        int n = rand(maxn) + 1;
        int m = rand(maxn) + 1;
        int x = rand(n) + 1;
        int y = rand(m) + 1;
        print(n, m, x, y);
        printDescT(String.format("Random: %d %d %d %d", n, m, x, y));
    }

    private void genManual(int n, int m, int x, int y) throws IOException {
        print(n, m, x, y);
        printDescT(String.format("Random: %d %d %d %d", n, m, x, y));
    }





    //end generators

}
