import java.io.*;
import java.util.*;

public class TestGen {

    private void shuffle(List<?> a) {
        Collections.shuffle(a, random);
    }

    class Test {
        public String descr;
        private int k;
        public Test(int k, String descr) {
            this.descr = descr;
            this.k = k;
        }

        int k() {
            return k;
        }

        int n() {
            return sets.size();
        }

        public List<List<Integer>> sets = new ArrayList<>();

        void newSet() {
            sets.add(new ArrayList<>());
        }

        void add(int x) {
            sets.get(sets.size() - 1).add(x);
        }

        void shuffle() {
            for (List<Integer> set : sets)
                TestGen.this.shuffle(set);
            TestGen.this.shuffle(sets);
        }
    }

    int testCount, subtaskCount;
    PrintWriter testsDesc;
    static Random random = new Random(0xCEEECCEE);

    public static final int GROUP1_N = 18;
    public static final int GROUP2_N = 2000;
    public static final int MAX_N = 100_000;//10^5
    public static final int MAX_SUM = 2_000_00;
    public static final int MAX_VAL = 1_000_000_000;

    private String folder;
    private String subtask;
    boolean isSamples = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public void print(Test test) throws IOException {
        if (subtaskCount == 1 && test.k == 1)
            return;
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);
        System.out.println("Generating test #" + testCount + ": " + test.descr);
        if (!isSamples)
            test.shuffle();

        out.println(test.n() + " " + test.k());
        for (List<Integer> set : test.sets) {
            out.print(set.size());
            for (Integer x : set)
                out.print(" " + x);
            out.println();
        }
        out.close();
    testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, test.descr));
    }

    public void genSamples() throws IOException {
        setFolder("preliminary");
        setSubtask("subtask0");
        isSamples = true;

        System.out.println("==== Generating samples ===");

        Test s1 = new Test(2, "sample1");
        s1.newSet();s1.add(1); s1.add(3); s1.add(4);
        s1.newSet();s1.add(5);s1.add(1);s1.add(2);
        s1.newSet();s1.add(4);
        print(s1);

        Test s2 = new Test(3, "sample2");
        s2.newSet();s2.add(2);s2.add(1);s2.add(3);
        s2.newSet();s2.add(4);s2.add(1);
        s2.newSet();s2.add(4);s2.add(2);s2.add(4);
        s2.newSet();s2.add(3);s2.add(2);s2.add(3);s2.add(3);
        s2.newSet();s2.add(5);s2.add(6);
        print(s2);

        testCount = 0;
        isSamples = false;
    }

    void genHandleTests() throws IOException {
    }

    interface GenGroupInterface {
        void gen() throws IOException;
    }

    public void genSubtask(int n, int points) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + subtaskCount;
        setSubtask(subtask);
        setFolder("tests");
        System.out.println("==== Generating group #" + subtaskCount + " ===");

        print(randEq(n, 0.5));
        print(randEq(n, 1.0));
        print(randEq(n, 0.8));

        print(randEq2(n, 0.5));
        print(randEq2(n, 1));
        print(randEq2(n, 0.8));

        print(genInner(n, -1));
        print(genInner(n, -1));
        print(genInner(n, -1));

        print(genInner(n - 1, 2));
        print(genInner(n - 1, 2));

        for (int i = 0; i < 5; ++i) {
            int ni = randInt(n);
            int ki = randInt(ni);
            print(genRandom(ni, ki));
        }

        print(genRandom(n, n));
        print(genRandom(n, n));
        print(genRandom(n, n));
        print(genRandomKEqualsOne(randInt(n / 10, n / 4)));
        print(genRandomKEqualsOne(randInt(n / 10, n / 4)));
        if (n > 1000)
            print(genRandomKEqualsOne(n / 1000));

        testsDesc.println(String.format("comment_subtask%d=\"n &lt;= %d\"", subtaskCount, n));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }

    private Test genRandomKEqualsOne(int n) {
        int[] part = genPartition(MAX_SUM - n, n);
        Test test = new Test(1, String.format("random: n = %d, k = %d, sum = %d", n, 1, MAX_SUM));
        for (int i = 0; i < n; ++i) {
            test.newSet();
            for (int j = 0; j < part[i]; ++j)
                test.add(randInt(MAX_VAL));
            test.add(randInt(MAX_VAL));
        }
        return test;
    }

    private Test randEq2(int n, double probably) {
        int k = randInt(n);
        List<Segment> segs = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            if (random.nextDouble() < probably) {
                int x = randInt(MAX_VAL);
                segs.add(new Segment(x, x));
            } else
                segs.add(Segment.randSegment());
        Test test = new Test(k, String.format("rand eq2: n = %d, k = %d", n, k));
        return inflate(test, segs);
    }

    private Test randEq(int n, double probably) {
        int x = randInt(MAX_VAL);
        int k = randInt(n);
        List<Segment> segs = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            if (random.nextDouble() < probably)
                segs.add(new Segment(x, x));
            else
                segs.add(Segment.randSegment());
        Test test = new Test(k, String.format("rand eq1: n = %d, k = %d", n, k));
        return inflate(test, segs);
    }

    public static class Segment {
        public static Segment MAX_SEGMENT = new Segment(1, MAX_VAL);
        int l, r;
        public Segment(int l, int r) {
            this.l = l;
            this.r = r;
        }

        public static Segment randSegment() {
            return randSegment(MAX_SEGMENT);
        }

        public static Segment randSegment(Segment s) {
            int l = randInt(s);
            int r = randInt(s);
            int t;
            if (l > r) {
                t = l;
                l = r;
                r = t;
            }
            return new Segment(l, r);
        }

        public static Segment randOuterSegment(Segment s) {
            int l = TestGen.randInt(1, s.l);
            int r = TestGen.randInt(s.r, MAX_VAL);
            return new Segment(l, r);
        }

        public static int randInt(Segment segment) {
            return TestGen.randInt(segment.l, segment.r);
        }
    }

    private Test genInner(int maxN, int coef) {
        int n = randInt(maxN);
        int k = randInt(n);
        Segment bigSegment = Segment.randSegment();
        List<Segment> segs = new ArrayList<>();
        int c = (coef == -1 ? coef - 1: coef * k);
        for (int i = 0; i < c; ++i)
            segs.add(Segment.randOuterSegment(bigSegment));
        for (int i = 0; i < n - c; ++i)
            segs.add(Segment.randOuterSegment(bigSegment));
        Test test = new Test(k, String.format("many inner segments n = %d, k = %d", n, k));
        return inflate(test, segs);
    }

    private Test inflate(Test test, List<Segment> segs) {
        int n = segs.size();
        int[] part = genPartitionWithZeroes(MAX_SUM - 2 * n, n);
        for (int i = 0; i < part.length; ++i) {
            test.newSet();
            test.add(segs.get(i).l);
            for (int j = 0; j < part[i]; ++j)
                test.add(Segment.randInt(segs.get(i)));
            test.add(segs.get(i).r);
        }
        return test;
    }

    private Test genRandom(int n, int k) {
        int sum = randInt(n, MAX_SUM);
        int[] part = genPartition(sum, n);
        Test test = new Test(k, String.format("random: n = %d, k = %d, sum = %d", n, k, sum));
        for (int i = 0; i < n; ++i) {
            test.newSet();
            for (int j = 0; j < part[i]; ++j)
                test.add(randInt(MAX_VAL));
        }
        return test;
    }

    private static int randInt(int l, int r) {
        return random.nextInt(r - l + 1) + l;
    }

    private static int randInt(int n) {
        return random.nextInt(n) + 1;
    }

    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genSamples();

        genSubtask(GROUP1_N, 31);
        genSubtask(GROUP2_N, 31);
        genSubtask(MAX_N, 38);
        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }

    private int[] genCombine(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genPartitionViaCombine(int n, int[] comb) {
        int k = comb.length + 1;
        int[] ret = new int[k];
        if (k >= 2)
            ret[0] = comb[0];

        for (int i = 1; i + 1 < k; ++i)
            ret[i] = comb[i] - comb[i - 1];
        ret[k - 1] = k >= 2 ?  n - comb[k - 2] : n;
        return ret;
    }

    public int[] genPartition(int sum, int k) {
        int[] comb = genCombine(sum - 1, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genPartitionWithZeroes(int sum, int k) {
        int[] comb = genCombineWithEq(sum, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genCombineWithEq(int n, int k) {
        int[] ret = new int[k];
        for (int i = 0; i < k; ++i)
            ret[i] = randInt(0, n);
        Arrays.sort(ret);
        return ret;
    }
}
