/**
 * Created by Grigorii Shovkoplias on 24.03.2016.
 */

import java.io.*;
import java.util.*;

public class TestGen {

    private void gen() throws Exception {
        new File("../tests").mkdir();
        new File("../preliminary").mkdir();
        desc = new PrintWriter("../tests.desc");


        done = 0;
        genSamples();

        //-------------------------first group-------------------------------------------------
        subtask = "subtask1";

        genManual();
        for (int i = 0; i <= 3; i++) {
            genRand(rand(MAXN1) + 1, MAXX1);
            genRand(gand(MAXN1), rand(MAXX1) + 1);
            genSame(gand(MAXN1), rand(MAXX1) + 1);
            genDif(rand(MAXN1) + 1);
        }

        genRand(MAXN1, MAXX1);
        genSame(MAXN1, rand(MAXX1) + 1);
        genSame(MAXN1, MAXX1);
        genDif(MAXN1);



        desc.println(String.format("comment_subtask%d=\"n &lt;= %d, l_i &lt;= %d\"", 1, MAXN1, MAXX1));
        desc.println(String.format("bonus_subtask%d=%d", 1, 33));

        //-------------------------second group-------------------------------------------------

        subtask = "subtask2";

        for (int i = 0; i <= 3; i++) {
            genRand(rand(MAXN2) + 1, MAXX2);
            genRand(gand(MAXN2), rand(MAXX2) + 1);
            genSame(gand(MAXN2), rand(MAXX2) + 1);
            genDif(rand(MAXN2) + 1);
        }

        genRand(MAXN2, MAXX2);
        genSame(MAXN2, rand(MAXX2) + 1);
        genSame(MAXN2, MAXX2);
        genDif(MAXN2);

        desc.println(String.format("comment_subtask%d=\"n, l_i &lt;= %d\"", 2, MAXN2));
        desc.println(String.format("bonus_subtask%d=%d", 2, 40));

        //-------------------------third group-------------------------------------------------

        subtask = "subtask3";

        for (int i = 0; i <= 3; i++) {
            genRand(rand(MAXN3) + 1, MAXX3);
            genRand(gand(MAXN3), rand(MAXX3) + 1);
            genSame(gand(MAXN3), rand(MAXX3) + 1);
            genDif(rand(MAXN3) + 1);
        }

        genRand(MAXN3, MAXX3);
        genSame(MAXN3, rand(MAXX3) + 1);
        genSame(MAXN3, MAXX3);
        genDif(MAXN3);

        desc.println(String.format("comment_subtask%d=\"n, l_i &lt;= %d\"", 3, MAXN3));
        desc.println(String.format("bonus_subtask%d=%d", 3, 27));


        desc.close();

    }

    public static void main(String[] args) throws Exception {
        new TestGen().gen();
    }

    private int done = 0;
    private PrintWriter desc;
    private final Random rnd = new Random(3289214075201L);
    private final int MAXN1 = 20;
    private final int MAXN2 = 100;
    private final int MAXN3 = 4000;
    private final int MAXX1 = 100;
    private final int MAXX2 = 100;
    private final int MAXX3 = 4000;

    private int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    private char randChar() {
        return (char) (rand(26) + 'a');
    }

    private int gand(int n) {
        return rand(n / 2) + n / 2 + 1;
    }

    private boolean coin() {
        return rnd.nextBoolean();
    }

    private long rand(long n) {
        if (n < 1)
            return 0;
        long res = rand(Integer.MAX_VALUE);
        res <<= 32;
        res += rand(Integer.MAX_VALUE);
        res %= n;
        return res;
    }


    private void print(int[] test) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", done));
        out.println(test.length);
        for (int i = 0; i < test.length - 1; i++) {
            out.print(test[i] + " ");
        }
        out.println(test[test.length - 1]);
        out.close();
    }


    private String getPath() {
        return String.format("../tests/%02d", done);
    }

    private void printDesc(String path, String folder, String subtask, int points, String comment) throws IOException {
        desc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, points, comment));
    }

    private String subtask;

    private void printDescT(String comment) throws IOException {
        printDesc(getPath(), "tests", subtask, 0, comment);
    }

    //begin generators

    private void genSamples() throws IOException {
        PrintWriter out;
        String path;
        path = "../preliminary/01";
        out = new PrintWriter(path);
        out.println("3");
        out.println("10 11 12");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 1");
        path = "../preliminary/02";
        out = new PrintWriter(path);
        out.println("4");
        out.println("100 1 2 3");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 2");
        path = "../preliminary/03";
        out = new PrintWriter(path);
        out.println("4");
        out.println("5 5 5 5");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 3");
    }


    private void shuffle(int[] a) {
        for (int i = 0; i < a.length; i++) {
            int j = rand(a.length);
            int tmp = a[j]
                    ;
            a[j] = a[i];
            a[i] = tmp;
        }
    }

    //begin generators

    private void genManual() throws IOException {
        print(new int[]{1});
        printDescT("1");
        print(new int[]{1, 2});
        printDescT("1 2");
        print(new int[]{10, 5, 5});
        printDescT("10 5 5");
        print(new int[]{3, 2, 1});
        printDescT("3 2 1");
    }

    private void genRand(int n, int maxLen) throws IOException {
        int[] test = new int[n];
        for (int i = 0; i < n; i++) {
            test[i] = rand(maxLen) + 1;
        }
        print(test);
        printDescT(String.format("Random: n = %d, maxLen = %d", n, maxLen));
    }

    private void genSame(int n, int x) throws IOException {
        int[] test = new int[n];
        for (int i = 0; i < n; i++) {
            test[i] = x;
        }
        print(test);
        printDescT(String.format("Same: n = %d, x = %d", n, x));
    }

    private void genDif(int n) throws IOException {
        int[] test = new int[n];
        for (int i = 0; i < n; i++) {
            test[i] = i + 1;
        }
        shuffle(test);
        print(test);
        printDescT(String.format("Different: n = %d", n));
    }





    //end generators

}
