import ru.ifmo.testlib.*;
import ru.ifmo.testlib.Outcome.Type;

public class Check implements Checker {

    public String readBooleanArray(int n, InStream in, boolean[] ret) {
        for (int i = 0; i < n; ++i) {
            int b = in.nextInt();
            if (b != 0 && b != 1)
                return String.format("a[%d] expected 0 or 1, but received %d", i, b);
            ret[i] = b == 1;
        }
        return null;
    }

    public boolean checkAnswer(int d, boolean[] seating, int[] good) {
        int n = seating.length;
        int cg = 0;
        for (int i = n - d; i < n; ++i)
            if (seating[i]) cg++;
        for (int i = 0; i <= d; ++i)
            if (seating[i]) cg++;
        if (cg != good[0])
            return false;
        for (int i = 1; i < n; ++i) {
            int nx = (i + d) % n;
            if (seating[nx]) cg++;

            int prv = (i - d + n - 1) % n;
            if (seating[prv]) cg--;
            if (cg != good[i])
                return false;
        }
        return true;
    }


    public boolean isYes(String s) {
        return s.equals("yes");
    }

    public boolean isNo(String s) {
        return s.equals("no");
    }

    private boolean validAns(String s) {
        return s.equals("yes") || s.equals("no");
    }

    @Override
    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        int n = inf.nextInt();
        int d = inf.nextInt();
        inf.nextLine();
        int[] good = new int[n];
        for (int i = 0; i < n; ++i)
            good[i] = inf.nextInt();

        String jAns = ans.nextToken().toLowerCase();
        String pAns = ouf.nextToken().toLowerCase();
        ans.nextLine();
        ouf.nextLine();

        if (!validAns(jAns))
            return new Outcome(Type.FAIL, "Jury answer != yes or no");
        if (!validAns(pAns))
            return new Outcome(Type.PE, "Part answer != yes or no");

        if (pAns.equals("no") && jAns.equals("no"))
            return new Outcome(Type.OK, "No solution.");

        boolean[] jSeating = new boolean[n], pSeating = new boolean[n];
        if (isYes(jAns)) {
            String invMessage = readBooleanArray(n, ans, jSeating);
            if (invMessage != null)
                return new Outcome(Type.FAIL, invMessage);
        }

        if (isYes(pAns)) {
            String invMessage = readBooleanArray(n, ouf, pSeating);
            if (invMessage != null)
                return new Outcome(Type.PE, invMessage);
        }

        if (isYes(jAns)) {
            if (!checkAnswer(d, jSeating, good))
                return new Outcome(Type.FAIL, "Jury has wrong certificate!");

            if (isNo(pAns))
                return new Outcome(Type.WA, "Jury found answer, but part didn't find!");
            else if (!checkAnswer(d, pSeating, good))
                return new Outcome(Type.WA, "Part has wrong certificate!");
        } else {
            if (isYes(pAns) && checkAnswer(d, pSeating, good))
                return new Outcome(Type.FAIL, "Jury hasn't answer, but part has!");
            else if (isYes(pAns))
                return new Outcome(Type.WA, "Part has wrong certificate!");
        }
        return new Outcome(Type.OK, String.format("Ok. n = %d", n));
    }
}

