import java.io.*;
import java.util.*;

public class TestGen {

    static class Test {
        public int[] a;
        public int d;
        public String descr;

        public Test(int[] a, int d, String descr) {
            this.a = a;
            this.d = d;
            this.descr = descr;
        }

        public int n() {
            return a.length;
        }
    }

    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(0xCEEECCEE);

    public static final int GROUP1_N = 18;
    public static final int GROUP2_N = 1000, GROUP2_D = 5;
    public static final int GROUP3_N = 2000;
    public static final int MAX_N = 100_000;//10^5

    private String folder;
    private String subtask;
    private int groupNo;
    boolean isSamples = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public void setGroupNo(int groupNo) {
        this.groupNo = groupNo;
    }

    private int[] calcBySeating(int[] seating, int d) {
        return calcBySeating(castToBoolean(seating), d);
    }

    public void print(Test test) throws IOException {
        String path = String.format("../%s/%03d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);
        System.out.println("Generating test #" + testCount + ": " + test.descr);

        out.println(test.a.length + " " + test.d);
        for (int i = 0; i < test.a.length; ++i)
            if (i + 1 == test.a.length)
                out.println(test.a[i]);
            else
                out.print(test.a[i] + " ");
        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, test.descr));
    }

    private boolean[] castToBoolean(int[] seating) {
        boolean[] ret = new boolean[seating.length];
        for (int i = 0; i < seating.length; ++i)
            if (seating[i] == 0)
                ret[i] = false;
            else if (seating[i] == 1)
                ret[i] = true;
            else
                throw new AssertionError("invalid cast of seating to boolean");
        return ret;
    }

    private int[] calcBySeating(boolean[] seating, int d) {
        int n = seating.length;
        int[] ret = new int[seating.length];
        int cg = 0;
        for (int i = 0; i <= d; ++i)
            if (seating[i]) cg++;
        for (int i = n - d; i < n; ++i)
            if (seating[i]) cg++;
        ret[0] = cg;

        for (int i = 1; i < n; ++i) {
            int nx = (i + d) % n;
            if (seating[nx]) cg++;

            int prv = (i - d + n - 1) % n;
            if (seating[prv]) cg--;
            ret[i] = cg;
        }
        return ret;
    }

    public void genSamples() throws IOException {
        setFolder("preliminary");
        setSubtask("subtask0");
        isSamples = true;

        System.out.println("==== Generating samples ===");
        print(new Test(new int[]{2, 2, 1, 2, 2}, 1, "sample1"));
        print(new Test(new int[]{3, 4, 4, 3, 3, 2, 2, 2, 2}, 2, "sample1"));
        print(new Test(new int[]{3, 3, 3, 3, 3, 1}, 1, "sample3"));
        testCount = 0;
        isSamples = false;
    }

    void genHandleTests() throws IOException {
    }

    public int gcd(int x, int y) {
        if (x == 0)
            return y;
        return gcd(y % x, x);
    }

    public Test genCyclic(int n, int d, double prob) {
        int g = gcd(n, 2 * d + 1);
        int[] a = new int[n];
        int step = 2 * d + 1;
        Arrays.fill(a, -1);

        for (int i = 0; i < g; ++i)
            if (random.nextDouble() < prob) {
                int r = i;
                int bit = random.nextInt(2);
                do {
                    a[r] = bit;
                    r = (r + step) % n;
                } while (r != i);
            } else {
                double probablyOne = random.nextDouble();
                int r = i;
                do {
                    a[r] = random.nextDouble() < probablyOne ? 1 : 0;
                    r = (r + step) % n;
                } while (r != i);
            }
        return new Test(calcBySeating(a, d), d, String.format("gen cyclic n = %d, d = %d, prob = %.2f, gcd = %d, (2d+1)/gcd = %d", n, d, prob, g, (2 * d + 1) / g));
    }

    interface GenGroupInterface {
        void gen() throws IOException;
    }

    public void genSubtask(int n, int d, int points, GenGroupInterface genGroup, int groupNo) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + subtaskCount;
        setSubtask(subtask);
        setFolder("tests");
        //setGroupNo(groupNo);
        System.out.println("==== Generating group #" + groupNo + " ===");
        genGroup.gen();

        testsDesc.println(String.format("comment_subtask%d=\"n &lt;= %d, d &lt;= %d\"", subtaskCount, n, d));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }

    private Test genFromRealData(int n, int d, double prob) {
        boolean[] seatings = new boolean[n];
        int g = gcd(n, 2 * d + 1);
        for (int i = 0; i < n; ++i)
                seatings[i] = random.nextDouble() < prob;
        return new Test(calcBySeating(seatings, d), d, String.format("gen from real data n = %d, d = %d, prob = %.2f, g = %d, (2d+1)/g = %d", n, d, prob, g, (2 * d + 1) / g));
    }

    private int randInt(int n) {
        return random.nextInt(n) + 1;
    }


    private void push(int[] a, int step, int startIndex, int value) {
        int n = a.length;
        a[startIndex] = value;
        for (int curIndex = (startIndex + step) % n;
             startIndex != curIndex;
             curIndex = (curIndex + step) % n )
            a[curIndex] = value;
    }

    private Test genCycleEqualsNo(int n, int d, int diff) {
        int step = 2 * d + 1;
        int g = gcd(n, step);
        int x = random.nextInt(2 * d + 2);
        int[] a = new int[n];
        Arrays.fill(a, -1);
        push(a, step, 0, x);
        for (int i = 0; i < n; ++i)
            if (a[i] == -1) {
                int prev = a[(i - 1 + n) % n];
                int lef = Math.max(0, prev - diff);
                int rig = Math.min(2 * d + 1, prev + diff);
                int val = randInt(lef, rig);
                push(a, step, i, val);
            }
        return new Test(a, d, String.format("cycle equals no n = %d, d = %d, gcd = %d, (2d+1)/g = %d, diff = %d", n, d, g, (2 * d + 1) / g, diff));
    }

    private int randInt(int lef, int rig) {
        return randInt(rig - lef + 1) + lef - 1;
    }


    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genSamples();
        genSubtask(GROUP1_N, (GROUP1_N - 1) / 2, 20, ()->{
            print(new Test(new int[]{3, 3, 3, 3, 3, 3}, 2, "handle test"));
            print(genFromRealData(6, 1, 0.5));
            print(genFromRealData(18, 1, 0.5));
            print(genFromRealData(18, 4, 0.3));
            print(genFromRealData(17, 4, 0.1));
            print(genFromRealData(17, 8, 0.0));
            print(genFromRealData(17, 7, 0.5));

            print(genCyclic(11, 4, 0.5));
            print(genCyclic(11, 5, 0.5));
            print(genCyclic(13, 3, 1));
            print(genCyclic(12, 2, 0.5));

            print(genNo(genFromRealData(18, 4, 0.5)));
            print(genNo(genFromRealData(17, 4, 0.5)));
            print(genNo(genFromRealData(14, 3, 1.0)));
            print(genNo(genCyclic(16, 4, 0.5)));
            print(genNo(genCyclic(17, 8, 0.5)));
            print(genRandom(12, 4));

            print(genCycleEqualsNo(18, 4, 1));
            print(genCycleEqualsNo(16, 5, 10));
            print(genCycleEqualsNo(8, 3, 1));
            print(genCycleEqualsNo(15, 2, 1));
            print(genAllEquals(17, 3));
            print(genAllEquals(13, 5));
        }, 1);

        genSubtask(GROUP2_N, GROUP2_D, 23, ()->{
            print(genFromRealData(999, 1, 0.5));
            print(genFromRealData(333, 4, 0.5));
            print(genFromRealData(960, 4, 0.5));
            print(genFromRealData(998, 3, 0.5));
            print(genFromRealData(231, 2, 0.5));
            print(genFromRealData(999, 5, 0.6));
            print(genFromRealData(987, 5, 0.5));

            print(genCyclic(921, 4, 0.5));
            print(genCyclic(921, 4, 1));
            print(genCyclic(1000, 1, 1.0));
            print(genCyclic(994, 3, 0.8));
            print(genCyclic(994, 3, 0.5));
            print(genCyclic(993, 3, 0.5));

            print(genNo(genFromRealData(945, 4, 0.5)));
            print(genNo(genFromRealData(1000, 3, 0.5)));
            print(genNo(genFromRealData(990, 5, 0.5)));
            print(genNo(genCyclic(991, 5, 0.5)));
            print(genNo(genCyclic(800, 2, 1.0)));

            print(genRandom(GROUP2_N, 1));
            print(genCycleEqualsNo(880, 5, 1));
            print(genCycleEqualsNo(995, 4, 1));
            print(genCycleEqualsNo(700, 1, 10));
            print(genAllEquals(620, 2));
        }, 2);

        genSubtask(GROUP3_N, (GROUP3_N - 1) / 2,  25, ()->{
            print(genFromRealData(1800, 4, 0.5));
            print(genFromRealData(237*8, 197, 0.5));
            print(genFromRealData(237*8, 197, 0.0));
            print(genFromRealData(1720, 150, 0.5));
            print(genFromRealData(1720, 150, 1.0));
            print(genFromRealData(607 * 3, 303, 0.7));

            print(genCyclic(1410, 305, 0.5));
            print(genCyclic(1427, 713, 0.5));
            print(genCyclic(1872, 358, 0.5));
            print(genCyclic(1872, 358, 1));
            print(genCyclic(1880, 117, 0.5));

            printTestsWithBigGCD(subtaskCount, GROUP3_N);

            print(genNo(genFromRealData(1720, 150, 1.0)));
            print(genNo(genFromRealData(607 * 3, 303, 0.7)));
            print(genNo(genCyclic(1880, 117, 0.5)));
            print(genNo(genCyclic(1880, 117, 1)));
            print(genRandom(GROUP3_N, 998));

            print(genAllZeroes(1872, 358));
            print(genAllOnes(1880, 117));
            print(genCycleEqualsNo(580, 14, 1));
            print(genCycleEqualsNo(992, 13, 1));
            print(genCycleEqualsNo(1850, 335, 1));
            print(genCycleEqualsNo(1777, 12, 1));
            print(genCycleEqualsNo(1777, 12, 20));
            print(genAllEquals(1998, 166));
        }, 3);

        genSubtask(MAX_N, (MAX_N - 1) / 4, 32, () -> {
            print(genFromRealData(MAX_N, (99989 - 1) / 2, 0.5));
            print(genFromRealData(MAX_N, (99989 - 1) / 2, 1.0));

            print(genFromRealData(97138, 9999, 0.5));
            print(genFromRealData(97138, 9999, 1));
            print(genFromRealData(91917, 13131, 0.5));

            print(genCyclic(97138, 9999, 1));
            print(genCyclic(91917, 13131, 0.5));

            printTestsWithBigGCD(subtaskCount, MAX_N);//TODO with big d/gcd
            print(genAllZeroes(91917, 13131));
            print(genAllOnes(91917, 13131));

            print(genNo(genCyclic(90150, 1, 1.0)));
            print(genNo(genFromRealData(91917, 13131, 0.5)));
            print(genRandom(MAX_N, MAX_N / 3));

            print(genCycleEqualsNo(66667, 33333, 1));
            print(genCycleEqualsNo(3*25*10*13*8, 38272/3, 1));
            print(genCycleEqualsNo(35*27*16*7/2, 38272/3, 1));
            print(genAllEquals(MAX_N, 19877));
        }, 4);

        testsDesc.close();
    }

    private Test genAllEquals(int n, int d) {
        int x = random.nextInt(2 * d + 2);
        int[] a = new int[n];
        for (int i = 0; i < n; ++i) a[i] = x;
        return new Test(a, d, "all equals " + x);
    }

    private Test genAllOnes(int n, int d) {
        int[] a = new int[n];
        for (int i = 0; i < n; ++i) a[i] = 2 * d + 1;
        return new Test(a, d, "all ones");
    }

    private Test genAllZeroes(int n, int d) {
        int[] a = new int[n];
        return new Test(a, d, "all zeroes");
    }

    private Test genRandom(int n, int d) {
        int[] a = new int[n];
        for (int i = 0; i < n; ++i)
            a[i] = random.nextInt(2 * d + 2);
        return new Test(a, d, "random: no");
    }

    private Test genNo(Test test) {
        if (random.nextBoolean()) {
            int id = random.nextInt(test.n());
            test.a[id] = random.nextInt(2 * test.d + 2);
            test.descr += ", ans = NO. fix one position";
            return test;
        } else {
            int n = test.n();
            int[] a = new int[n];
            int d = test.d;
            int id = random.nextInt(test.n());
            a[id] = random.nextInt(2 * d + 2);
            int i = (id + 1) % n;
            while (i != id) {
                int diff = random.nextInt(3) - 1;
                while (a[(i + n - 1) % n] + diff < 0 || a[(i + n - 1) % n] + diff > 2 * d + 1)
                    diff = random.nextInt(3) - 1;
                a[i] = a[(i + n - 1) % n] + diff;
                i = (i + 1) % n;
            }
            return new Test(a, d, "ans = NO. rand inc, dec");
        }
    }

    private void printTestsWithBigGCD(int groupNo, int maxN) throws IOException {
        int g = 11;
        int ml = 2 * random.nextInt(maxN / g / 2) + 1;
        int d2 = ml * g;
        int d = (d2 - 1) / 2;

        print(genCyclic(randInt(maxN / d2) * d2, d, 0.5));


        g = 13131;
        if (maxN > g) {
            ml = 2 * random.nextInt(maxN / g / 2) + 1;
            d2 = ml * g;
            d = (d2 - 1) / 2;
            print(genCyclic(randInt(maxN / d2) * d2, d, 0.2));
        }

        if (groupNo == 3) {
            print(genWithGCD(maxN, 103, 0.5));
            print(genWithGCD(maxN, 333, 0.5));

            print(genWithGCD(maxN, 211, 0.8));
        } else if (groupNo == 4) {
            print(genCyclic(89100, 742, 0.5));
            print(genCyclic(89100, 742, 1));
            print(genCyclic(98280, 33196, 0.5));
            print(genFromRealData(98280, 33196, 0.5));
            print(genFromRealData(99388, 19314, 0.5));
            print(genCyclic(98415, 9841, 0.5));
            print(genCyclic(97202, 24300, 0.5));
            print(genCyclic(3*5*10*25*16, 19966, 1));
            print(genCyclic(3 * 5 * 10 * 25 * 16, 19966, 0));

            print(genCyclic(35*27*16*7/2, 38272 / 3, 0.5));
            print(genCyclic(35*27*16*7/2, 38272/3, 1.0));


            /*print(genWithGCD(maxN, 999, 0.5));
            print(genWithGCD(maxN, 23333, 0.5));
            print(genWithGCD(maxN, 23333, 1));
            print(genWithGCD(maxN, 3277, 0.5));
            print(genWithGCD(maxN, 3277, 1));
            print(genWithGCD(maxN, 5093, 0.5));*/
        }
    }

    private Test genWithGCD(int maxN, int g, double prob) {
        if (g % 2 == 0)
            throw new IllegalArgumentException("g % 2 == 0");
        int d = 2 * randInt(maxN / g / 4) + 1;
        List<Integer> candidates = new ArrayList<>();
        for (int i = 1; i <= maxN; ++i)
            if (i >= 2 * d + 1 && i % g == 0)
                candidates.add(i);
        shuffle(candidates);
        int n = candidates.get(0);
        return genCyclic(n, d, prob);
    }

    private void shuffle(List<?> a) {
        Collections.shuffle(a, random);
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}